#include "marinebleed.h"
#include "mtrand.h"


//
// Global variables
//
short g_sModelIndexBloodSpray;
short g_sModelIndexBloodDrop;

enum decal_e 
{	
	DECAL_BLOOD1 = 0, 
	DECAL_BLOOD2, 
	DECAL_BLOOD3, 
	DECAL_BLOOD4, 
	DECAL_BLOOD5, 
	DECAL_BLOOD6, 
};

DLL_DECALLIST gDecals[6] = {
	{ "{blood1", 0 },		// DECAL_BLOOD1
	{ "{blood2", 0 },		// DECAL_BLOOD2
	{ "{blood3", 0 },		// DECAL_BLOOD3
	{ "{blood4", 0 },		// DECAL_BLOOD4
	{ "{blood5", 0 },		// DECAL_BLOOD5
	{ "{blood6", 0 }		// DECAL_BLOOD6
};


//
// Static variables
//
MTRandClass RandGen(0x01071981);


//
// Static functions
//
static void UTIL_DecalTrace(TraceResult *pTrace)
{
	short entityIndex;
	int index;
	int message;

	index = gDecals[DECAL_BLOOD1 + RandGen.RandomUnsignedLong(5)].index;
	if(index < 0 || pTrace->flFraction > 0.999f)
		return;

	// Only decal BSP models
	if(pTrace->pHit)
	{
		const char * str = STRING(pTrace->pHit->v.model);
		if(str && str[0]=='*')
			return;
		entityIndex = ENTINDEX( pTrace->pHit );
	}
	else
	{
		entityIndex = 0;
	}

	message = TE_DECAL;
	if(entityIndex != 0)
	{
		if(index > 255)
		{
			message = TE_DECALHIGH;
			index -= 256;
		}
	}
	else
	{
		message = TE_WORLDDECAL;
		if( index > 255)
		{
			message = TE_WORLDDECALHIGH;
			index -= 256;
		}
	}
	
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
		WRITE_BYTE( message );
		WRITE_COORD( pTrace->vecEndPos.x );
		WRITE_COORD( pTrace->vecEndPos.y );
		WRITE_COORD( pTrace->vecEndPos.z );
		WRITE_BYTE( index );
		if ( entityIndex )
			WRITE_SHORT( entityIndex );
	MESSAGE_END();
}


//
// Globals functions
//
void SpawnBlood(const Vector &vecSpot, float flDamage)
{
	if(flDamage<=0)
		return;
	
	int amount = (int)(flDamage * marinebleed_gorelevel->value * 2.0f);
	
	if(amount > 255)
		amount = 255;
	
	MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, vecSpot );
		WRITE_BYTE( TE_BLOODSPRITE );
		WRITE_COORD( vecSpot.x );			// pos
		WRITE_COORD( vecSpot.y );
		WRITE_COORD( vecSpot.z );
		WRITE_SHORT( g_sModelIndexBloodSpray );		// initial sprite model
		WRITE_SHORT( g_sModelIndexBloodDrop );		// droplet sprite models
		WRITE_BYTE( BLOOD_COLOR_RED );			// red
		WRITE_BYTE( min( max( 3, amount / 10 ), 16 ) );	// size
	MESSAGE_END();
}

void TraceBleed(float flDamage, const Vector &vecStart, const Vector &vecDir, CPlayer *pVictim)
{
	if(flDamage<=0)
		return;
		
	// make blood decal on the wall! 
	TraceResult Bloodtr;
	Vector vecTraceDir; 
	
	float flNoise = marinebleed_bloodspread->value;
	int cCount = (int)marinebleed_gorelevel->value;
	
	if (flDamage < 10.0f)
	{
		flNoise *= 0.1f;
		cCount *= 1;
	}
	else if (flDamage < 25.0f)
	{
		flNoise *= 0.2f;
		cCount *= 2;
	}
	else if (flDamage < 75.0f)
	{
		flNoise *= 0.3f;
		cCount *= 4;
	}
	else
	{
		flNoise *= 0.5f;
		cCount *= 6;
	}

	for(int i = 0; i < cCount; i++)
	{
		vecTraceDir = vecDir * -1.0f;// trace in the opposite direction the shot came from (the direction the shot is going)

		vecTraceDir.x += RandGen.RandomFloat( -flNoise, flNoise );
		vecTraceDir.y += RandGen.RandomFloat( -flNoise, flNoise );
		vecTraceDir.z += RandGen.RandomFloat( -flNoise, flNoise );

		UTIL_TraceLine( vecStart, vecStart + vecTraceDir * -172.0f, ignore_monsters, pVictim->edict(), &Bloodtr);

		if(Bloodtr.flFraction < 0.999f)
		{
			UTIL_DecalTrace(&Bloodtr);
		}
	}
}
