#include "marinebleed.h"

//
// Static variables
//
static int       InCheck;
static Vector    AttackerOrigin;
static Vector    AttackerAngles;
static Vector    AttackerEyePos;
static edict_t * pAttackerEntity;


//
// Direct player attack check
//
static void PlayerPreThink( edict_t *pAttacker )
{
	InCheck = 0;
	
	//Can this cause damage to anyone?
	if(marinebleed_on->value!=0.0f && UTIL_MightCauseDamage(pAttacker))
	{
		InitializeDamageCheck(pAttacker);
		InCheck         = 1;
		pAttackerEntity = pAttacker;
		AttackerOrigin  = pAttacker->v.origin;
		AttackerEyePos  = AttackerOrigin + pAttacker->v.view_ofs;
		AttackerAngles  = pAttacker->v.v_angle;
	}
	
	RETURN_META(MRES_IGNORED);
}
static void PlayerPostThink_Post( edict_t *pAttacker )
{
	//Check for damage
	if(InCheck)
		CheckDamageAndSpreadBlood(pAttackerEntity, AttackerOrigin, AttackerAngles, AttackerEyePos, true);
	
	InCheck         = 0;
	pAttackerEntity = NULL;

	RETURN_META(MRES_IGNORED);
}


//
// touch entity attack check (missile hitting wall -> explode .. etc)
//
static void EntTouch( edict_t *pAttacker, edict_t* )
{
	InCheck = 0;
	
	//Can this cause damage to anyone?
	if(marinebleed_on->value!=0.0f && UTIL_MightCauseDamage(pAttacker))
	{
		InitializeDamageCheck(pAttacker);
		InCheck         = 1;
		pAttackerEntity = pAttacker;
		AttackerOrigin  = pAttacker->v.origin;
	}
	
	RETURN_META(MRES_IGNORED);
}
static void EntTouch_Post( edict_t *pAttacker, edict_t* )
{
	//Check for damage
	if(InCheck)
		CheckDamageAndSpreadBlood(pAttackerEntity, AttackerOrigin, AttackerAngles, AttackerEyePos, false);
	
	InCheck         = 0;
	pAttackerEntity = NULL;

	RETURN_META(MRES_IGNORED);
}


//
// timed entity attack check (laser mine?)
//
static void EntThink( edict_t *pAttacker )
{
	InCheck = 0;
	
	//Can this cause damage to anyone?
	if(marinebleed_on->value!=0.0f && UTIL_MightCauseDamage(pAttacker))
	{
		InitializeDamageCheck(pAttacker);
		InCheck         = 1;
		pAttackerEntity = pAttacker;
		AttackerOrigin  = pAttacker->v.origin;
	}
	
	RETURN_META(MRES_IGNORED);
}
static void EntThink_Post( edict_t *pAttacker )
{
	//Check for damage
	if(InCheck)
		CheckDamageAndSpreadBlood(pAttackerEntity, AttackerOrigin, AttackerAngles, AttackerEyePos, false);
	
	InCheck         = 0;
	pAttackerEntity = NULL;

	RETURN_META(MRES_IGNORED);
}


//
// Client handling
//
static void ClientPutInServer( edict_t *pEdict )
{
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(pEdict);
	
	// If index is 0 pThis is invalid :(
	// Don't think proxies as players
	if(pPlayer==&players[0] || pEdict->v.flags & FL_PROXY)
		pPlayer->pev = NULL;
	else
		pPlayer->pev = VARS(pEdict);
	
	RETURN_META(MRES_HANDLED);
}

static void ClientDisconnect( edict_t *pEdict )
{
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(pEdict);
	
	pPlayer->pev = NULL;
	
	RETURN_META(MRES_HANDLED);
}


//
// Level startup .. initialize level data here
//
static int DispatchSpawn( edict_t *pent )
{	
	if (stricmp(STRING(pent->v.classname), "worldspawn"))
		RETURN_META_VALUE(MRES_IGNORED, 0);
	
	//precache blood sprites
	g_sModelIndexBloodSpray = PRECACHE_MODEL("sprites/bloodspray.spr"); // initial blood
	g_sModelIndexBloodDrop = PRECACHE_MODEL("sprites/blood.spr"); // splattered blood 
		
	//precache decals
	for(unsigned i = 0; i < ARRAYSIZE(gDecals); i++)
		gDecals[i].index = DECAL_INDEX(gDecals[i].name);
		
	//Clear players
	memset(players, 0, sizeof(players));
	
	RETURN_META_VALUE(MRES_HANDLED,	0);
}


//
// dllapi interface functions
//
int __GetEntityAPI2(DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion)
{
	if(!pFunctionTable) 
	{
		return(FALSE);
	}
	else if(*interfaceVersion != INTERFACE_VERSION)	
	{
		//! Tell engine	what version we	had, so	it can figure out who is out of	date.
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}
	
	memset(pFunctionTable, 0, sizeof(DLL_FUNCTIONS));
	pFunctionTable->pfnPlayerPreThink    = PlayerPreThink;
	pFunctionTable->pfnClientDisconnect  = ClientDisconnect;
	pFunctionTable->pfnClientPutInServer = ClientPutInServer;
	pFunctionTable->pfnTouch             = EntTouch;
	pFunctionTable->pfnThink             = EntThink;
	pFunctionTable->pfnSpawn             = DispatchSpawn;

	if(!gDecals[0].name)
	{
		EntTouch(0,0);
		EntThink(0);
		PlayerPreThink(0);
		ClientDisconnect(0);
		ClientPutInServer(0);
		DispatchSpawn(0);
	}

	return(TRUE);
}

int __GetEntityAPI2_Post(DLL_FUNCTIONS *pFunctionTable_Post, int *interfaceVersion)
{
	if(!pFunctionTable_Post)
	{
		return(FALSE);
	}
	else if(*interfaceVersion != INTERFACE_VERSION)	
	{
		//! Tell engine	what version we	had, so	it can figure out who is out of	date.
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}
	
	memset(pFunctionTable_Post, 0, sizeof(DLL_FUNCTIONS));
	pFunctionTable_Post->pfnThink           = EntThink_Post;
	pFunctionTable_Post->pfnTouch           = EntTouch_Post;
	pFunctionTable_Post->pfnPlayerPostThink = PlayerPostThink_Post;
	
	if(!gDecals[0].name)
	{
		EntTouch_Post(0,0);
		EntThink_Post(0);
		PlayerPostThink_Post(0);
	}

	return(TRUE);
}
