#include "marinebleed.h"

//
// Global variables
//
CPlayer  players[33]; //players array .. index 0 is used for invalid entities
cvar_t * marinebleed_on;
cvar_t * marinebleed_gorelevel;
cvar_t * marinebleed_bloodspread;


//
// Static variables
//
static cvar_t marinebleed_cvars[] = {
	{"marinebleed_on",          "1", FCVAR_EXTDLL, 1, NULL},
	{"marinebleed_gorelevel",   "1", FCVAR_EXTDLL, 1, NULL},
	{"marinebleed_bloodspread", "1", FCVAR_EXTDLL, 1, NULL},

	{"marinebleed_version",VVERSION,FCVAR_EXTDLL|FCVAR_SERVER,0,NULL},
	
	{NULL,NULL,0,0,NULL},
};


//
// Damage monitor
//
void InitializeDamageCheck(edict_t *pAttacker)
{
	int count = gpGlobals->maxClients;
	for(int i = 1; i <= count; i++)
	{
		CPlayer * pPlayer = &players[i];
		pPlayer->m_InCheck = 0;
		
		//Do we check for damage for this player
		if(!pPlayer->IsValid() ||
		   !pPlayer->IsAlive() || 
		    pPlayer->IsSamePEV(pAttacker) || 
		   !pPlayer->IsMarine())
			continue;
		
		pPlayer->m_InCheck = 1;
		pPlayer->SaveHealth();
		pPlayer->SaveOrigin();
	}
}

void CheckDamageAndSpreadBlood(edict_t *pAttacker, const Vector &AttackerOrigin, const Vector &AttackerAngles, const Vector &AttackerEyePos, int UseAngles)
{
	//if it's null make sure traceline knows it too
	if(FNullEnt(pAttacker))
		pAttacker = NULL;

	int count = gpGlobals->maxClients;
	for(int i = 1; i <= count; i++)
	{
		CPlayer * pPlayer = &players[i];
		
		int incheck = pPlayer->m_InCheck;
		pPlayer->m_InCheck = 0;
		
		//does player have old health recorded
		if(!incheck || !pPlayer->IsValid())
			continue;
		
		//Check damage caused by attacker
		float flDamage = pPlayer->GetDamage();
		if(flDamage <= 0)
			continue;

		//Player has died.. make him solid for attack trace
		bool NotSolid = (pPlayer->pev->solid == SOLID_NOT);
		pPlayer->pev->solid = SOLID_BBOX;

		TraceResult Trace;
		
		//this is for direct player attack
		if(UseAngles && pAttacker)
		{
			UTIL_MakeVectors(AttackerAngles);
			
			//Extra accurancy needed because 16284.0 is big number
			double EndPos[3];
			EndPos[0] = (double)AttackerEyePos.x + ((double)gpGlobals->v_forward.x * (double)16384.0);
			EndPos[1] = (double)AttackerEyePos.y + ((double)gpGlobals->v_forward.y * (double)16384.0);
			EndPos[2] = (double)AttackerEyePos.z + ((double)gpGlobals->v_forward.z * (double)16384.0);
			
			//Trace blood drip point by using angles
			UTIL_TraceLine(AttackerEyePos, Vector(EndPos[0], EndPos[1], EndPos[2]), dont_ignore_monsters, pAttacker, &Trace);

			if(pPlayer->IsSamePEV(Trace.pHit))
			{
				Vector vecDir = (Trace.vecEndPos - AttackerEyePos).Normalize();

				//bloooood :o
				SpawnBlood(Trace.vecEndPos, flDamage);
				TraceBleed(flDamage, Trace.vecEndPos, vecDir, pPlayer);
				
				if(NotSolid)
					pPlayer->pev->solid = SOLID_NOT;
				
				continue;
			}
		}
		
		//Trace blood drip point
		UTIL_TraceLine(AttackerOrigin, pPlayer->GetOrigin(), dont_ignore_monsters, pAttacker, &Trace);
		
		if(pPlayer->IsSamePEV(Trace.pHit))
		{
			//Traced to player
			if(Trace.flFraction > 0.999f)
				Trace.vecEndPos = pPlayer->GetOrigin(); //shouldn't happen
			
			Vector vecDir = (pPlayer->GetOrigin() - AttackerOrigin).Normalize();
			
			//hm.. something red here .. OMG .. IT'S BLOOOOOOD!!!!
			SpawnBlood(Trace.vecEndPos, flDamage);
			TraceBleed(flDamage, Trace.vecEndPos, vecDir, pPlayer);

			if(NotSolid)
				pPlayer->pev->solid = SOLID_NOT;

			continue;
		}
		else
		{
			//Trace misses player
			Vector vecDir;
			if(UseAngles)
				vecDir = (pPlayer->GetOrigin() - AttackerEyePos).Normalize();
			else
				vecDir = (pPlayer->GetOrigin() - AttackerOrigin).Normalize();
			
			//ouch.. damn I'm bleeding!
			SpawnBlood(pPlayer->GetOrigin(), flDamage);
			TraceBleed(flDamage, pPlayer->GetOrigin(), vecDir, pPlayer);

			if(NotSolid)
				pPlayer->pev->solid = SOLID_NOT;

			continue;
		}
	}
}


//
// Plugin startup.
//
void plugin_init(void) 
{
	LOG_MESSAGE(PLID, "%s v%s, %s", Plugin_info.name, Plugin_info.version, Plugin_info.date);
	LOG_MESSAGE(PLID, "by %s", Plugin_info.author);
	LOG_MESSAGE(PLID, "   %s", Plugin_info.url);
	LOG_MESSAGE(PLID, "compiled: "__DATE__ "," __TIME__" EET");
	
	//Register list of cvars
	for(int i = 0; marinebleed_cvars[i].name; i++ )
		CVAR_REGISTER( &marinebleed_cvars[i] );

	marinebleed_on          = CVAR_GET_POINTER(marinebleed_cvars[0].name);
	marinebleed_gorelevel   = CVAR_GET_POINTER(marinebleed_cvars[1].name);
	marinebleed_bloodspread = CVAR_GET_POINTER(marinebleed_cvars[2].name);
}

