/*
 * Copyright  2001-2002 Jussi Kivilinna <hullu@unitedadmins.com>
 *
 *    This file is part of marinebleed Metamod Plugin.
 *
 *    You are allowed to copy/modify this file freely. I don't care :)
 */

#include "marinebleed.h"


// Must	provide	at least one of	these..
static META_FUNCTIONS gMetaFunctionTable = 
{
	NULL,			// pfnGetEntityAPI			HL SDK;		called before game DLL
	NULL,			// pfnGetEntityAPI_Post			META;		called after game DLL
	__GetEntityAPI2,	// pfnGetEntityAPI2			HL SDK2;	called before game DLL
	__GetEntityAPI2_Post,	// pfnGetEntityAPI2_Post		META;		called after game DLL
	NULL,			// pfnGetNewDLLFunctions		HL SDK2;	called before game DLL
	NULL,			// pfnGetNewDLLFunctions_Post		META;		called after game DLL
	NULL,			// pfnGetEngineFunctions		META;		called before HL engine
	NULL,			// pfnGetEngineFunctions_Post		META;		called after HL	engine
};


// Holds engine functionality callbacks
enginefuncs_t g_engfuncs;
globalvars_t  *gpGlobals;


// Receive engine function table from engine.
// This appears to be the _first_ DLL routine called by the engine, so we
// do some setup operations here.
#ifdef _WIN32
	C_DLLEXPORT void WINAPI GiveFnptrsToDll
#else
	C_DLLEXPORT void GiveFnptrsToDll
#endif
(enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals)
{
	g_engfuncs= *pengfuncsFromEngine;
	gpGlobals = pGlobals;
}


// Description of plugin.
// (V* info from vers_plugin.h)
plugin_info_t Plugin_info = {
	META_INTERFACE_VERSION, // ifvers
	VNAME,			// name
	VVERSION,		// version
	VDATE,			// date
	VAUTHOR,		// author
	VURL,			// url
	VLOGTAG,		// logtag
	PT_CHANGELEVEL,		// loadable
	PT_STARTUP,		// unloadable
};


// Global variables from metamod.  These variable names are referenced by
// various macros.
meta_globals_t *gpMetaGlobals;		// metamod globals
gamedll_funcs_t *gpGamedllFuncs;	// gameDLL function tables
mutil_funcs_t *gpMetaUtilFuncs;		// metamod utility functions


// Metamod requesting info about this plugin
//  ifvers			(given) interface_version metamod is using
//  pPlugInfo		(requested) struct with info about plugin
//  pMetaUtilFuncs	(given) table of utility functions provided by metamod
// Metamod requesting info about this plugin
//  ifvers		(given) interface_version metamod is using
//  pPlugInfo		(requested) struct with info about plugin
//  pMetaUtilFuncs	(given) table of utility functions provided by metamod
C_DLLEXPORT int Meta_Query(char *ifvers, plugin_info_t **pPlugInfo, mutil_funcs_t *pMetaUtilFuncs) 
{
	// Check for valid pMetaUtilFuncs before we continue.
	if(!pMetaUtilFuncs) 
	{
		return(FALSE);
	}
	gpMetaUtilFuncs=pMetaUtilFuncs;

	// Give metamod our plugin_info struct.
	*pPlugInfo=&Plugin_info;
	Plugin_info.ifvers=strdup(ifvers);

	return(TRUE);
}


// Metamod attaching plugin to the server.
//  now				(given) current phase, ie during map, during changelevel, or at startup
//  pFunctionTable	(requested) table of function tables this plugin catches
//  pMGlobals		(given) global vars from metamod
//  pGamedllFuncs	(given) copy of function tables from game dll
C_DLLEXPORT int Meta_Attach(PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs)
{
	if(now > Plugin_info.loadable) 
	{
		return(FALSE);
	}
	if(!pMGlobals) 
	{
		return(FALSE);
	}
	gpMetaGlobals=pMGlobals;
	if(!pFunctionTable) 
	{
		return(FALSE);
	}
	memcpy(pFunctionTable, &gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs=pGamedllFuncs;
	// Let's go.
	plugin_init();
	return(TRUE);
}


// Metamod detaching plugin from the server.
// now		(given) current phase, ie during map, etc
// reason	(given) why detaching (refresh, console unload, forced unload, etc)
C_DLLEXPORT int Meta_Detach(PLUG_LOADTIME now, PL_UNLOAD_REASON reason) 
{
	if(now > Plugin_info.unloadable && reason != PNL_CMD_FORCED) 
	{
		return(FALSE);
	}
	
	return(TRUE);
}


#ifdef _WIN32
BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved){return TRUE;}
#endif

